#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for writing out the linkage matrix, as well as the statistics.

@author: mark
"""

import logging
import scipy.spatial.distance as scidist
import csv

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def write_linkage_matrix(linkage_matrix, filename):
    """This writes a linkage matrix to file.
    """
    with open(filename, 'w') as outfile:
        csv_writer = csv.writer(outfile, delimiter='\t',
                                quoting=csv.QUOTE_NONE)
        csv_writer.writerow(["Cluster ID1", "Cluster ID2", "Distance", "Number of observations"])
        for row_number in range(linkage_matrix.shape[0]):
            csv_writer.writerow(create_linkage_file_line(linkage_matrix[row_number]))
    return 0

def create_linkage_file_line(linkage_matrix_row):
    """This formats the matrix elements for reproducible writing.
    """
    return ["{:.0f}".format(linkage_matrix_row[0]),
            "{:.0f}".format(linkage_matrix_row[1]),
            "{:.7f}".format(linkage_matrix_row[2]),
            "{:.0f}".format(linkage_matrix_row[3])]

def write_cophenetic_and_inconsistency(cophenetic_coefficient,
                                       cophenetic_distances,
                                       inconsistency_matrix, filename):
    """This writes out the cophenetic and inconsistency information.
    """
    cophenetic_lines = create_cophenetic_lines(cophenetic_coefficient, cophenetic_distances)
    inconsistency_lines = create_inconsistency_lines(inconsistency_matrix)
    with open(filename, 'w') as outfile:
        csv_writer = csv.writer(outfile, delimiter='\t',
                                quoting=csv.QUOTE_NONE)
        csv_writer.writerows(cophenetic_lines)
        csv_writer.writerows(inconsistency_lines)
    return 0
        

def create_cophenetic_lines(cophenetic_coefficient, cophenetic_distances):
    """This creates a list of lists containing the cophenetic coefficient and
    the cophenetic distances. 
    """
    cophenetic_information = []
    cophenetic_coefficient_line = ["Cophenetic coefficient", "{:.7f}".format(cophenetic_coefficient)]
    cophenetic_information.append(cophenetic_coefficient_line)
    cophenetic_information.append(["Cophenetic Distance Matrix"])
    square_form_cophenetic_dists = scidist.squareform(cophenetic_distances)
    for i in range(square_form_cophenetic_dists.shape[0]):
        cophenetic_information.append(create_cophenetic_distance_line(square_form_cophenetic_dists[i]))
    return cophenetic_information
    
def create_cophenetic_distance_line(cophenetic_distance_line):
    """This creastes a line from the symmetric square distance matrix.
    """
    return ["{:.7f}".format(cophenetic_distance_line[i]) for i in range(cophenetic_distance_line.shape[0])]

def create_inconsistency_lines(inconsistency_matrix):
    """This creates a list of lists containing the inconsistency matrix
    information ready for writing to file.
    """
    inconsistency_information = []
    header_line = ["Mean Heights of links", "SD for links", "Number of links", "Inconsistency coefficient"]
    inconsistency_information.append(header_line)
    for i in range(inconsistency_matrix.shape[0]):
        inconsistency_information.append(create_inconsistency_line(inconsistency_matrix[i]))
    return inconsistency_information
    
def create_inconsistency_line(inconsistency_matrix_line):
    """This formats the information in the inconsistency line for writing out.
    """
    return ["{:.7f}".format(inconsistency_matrix_line[0]),
            "{:.7f}".format(inconsistency_matrix_line[1]),
            "{:.1f}".format(inconsistency_matrix_line[2]),
            "{:.7f}".format(inconsistency_matrix_line[3])]
