#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for reading in the polynomial coefficients, and also the fractional
occupancy, and generating the solvent Maps. Temperature and the ranges of the
two SSIPs forthe x and y axes need to be supplied.

@author: mark
"""

import logging
import resultsanalysis.resultsoutput.plottinginput as plottinginput
import solventmapcreator.io.polynomialdatareader as polynomialdatareader
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator
import solventmapcreator.solvationcalculation.solvationplotinformation as solvationplotinformation


logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def create_solvation_plot(epsilon_i_list, epsilon_j_list,
                          solvent_filename, solvent_id,
                          polynomial_filename, polynomial_order, **kwargs):
    """This creates and outputs the solvation plot.
    """
    input_data = create_plot_input_data_from_files(epsilon_i_list, epsilon_j_list,
                                                   solvent_filename, solvent_id,
                                                   polynomial_filename,
                                                   polynomial_order, **kwargs)
    return plot_and_write_contour_map(input_data, **kwargs)

def plot_and_write_contour_map(input_data, **kwargs):
    """This creates and plots a contour map wit the input given.
    """
    return plottinginput.create_and_write_contour_plot(input_data, kwargs.get("fileformat", "eps"))

def create_plot_input_data_from_files(epsilon_i_list, epsilon_j_list,
                                      solvent_filename, solvent_id,
                                      polynomial_filename, polynomial_order, **kwargs):
    """This reads in the information and generates the data for plotting.
    """
    theta = calculate_fractional_occupancy_for_phase(solvent_filename, solvent_id, kwargs.get("soluteconcentration", 0.001))
    polynomial_coefficients = parse_polynomial_data_file_extract_polynomial_coefficients(polynomial_filename, polynomial_order)
    temperature = kwargs.get("temperature", 298.0)
    x_label = kwargs.get("x_label", r"$\beta$")
    y_label = kwargs.get("y_label", r"$\alpha$")
    input_data = create_plot_input_data(epsilon_i_list, epsilon_j_list,
                                        temperature, theta, polynomial_coefficients,
                                        x_label, y_label)
    append_output_filename_stem(input_data, solvent_id, **kwargs)
    return input_data

def append_output_filename_stem(plot_input_data, solvent_id, **kwargs):
    """This generates the outputfilename stem for the plot, and appends it to
    the plotting input data.
    """
    plot_input_data['figure_label'] = create_output_filename_stem(solvent_id, kwargs.get("directory", ""))

def create_output_filename_stem(solvent_id, directory):
    """This generates the outputfilename stem for the plot.
    """
    if directory != '':
        return directory + '/' + solvent_id.replace(',', '_')  + '_solv_map'
    else:
        return solvent_id.replace(',', '_')  + '_solv_map'

def create_plot_input_data(epsilon_i_list, epsilon_j_list, temperature, theta,
                           polynomial_coefficients, x_label, y_label):
    """This creates a dictionary containing the input data and labels for plotting.
    """
    return solvationplotinformation.create_plot_input_data(epsilon_i_list,
                                                           epsilon_j_list,
                                                           temperature, theta,
                                                           polynomial_coefficients,
                                                           x_label, y_label)

def calculate_fractional_occupancy_for_phase(solvent_filename, solvent_id,
                                             solute_concentration):
    """This calculates the fractional occupancy of the phase. This is the sum
    of the solvent and solute contributions.
    """
    return fractionaloccupancycalculator.calculate_fractional_occupancy_for_phase(solvent_filename,
                                                                                  solvent_id,
                                                                                  solute_concentration)

def parse_polynomial_data_file_extract_polynomial_coefficients(filename,
                                                               polynomial_order):
    """This parses a polynomial file and gets the coefficients for the given
    polynomial order.
    """
    return parse_polynomial_data_file(filename)[polynomial_order]["coefficients"]

def parse_polynomial_data_file(filename):
    """This parses a polynomial data file, and extracts the information
    relating to the polynomial fits, returning a dictionary.
    """
    return polynomialdatareader.parse_polynomial_data_file(filename)
